@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Modern; # git description: v0.576-7-gabcce010
+package JSON::Schema::Modern; # git description: v0.577-6-g4ee675c2
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema
 # KEYWORDS: JSON Schema validator data validation structure specification
 
-our $VERSION = '0.577';
+our $VERSION = '0.578';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -16,7 +16,7 @@ use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use JSON::MaybeXS;
+use Mojo::JSON ();  # for JSON_XS, MOJO_NO_JSON_XS environment variables
 use Carp qw(croak carp);
 use List::Util 1.55 qw(pairs first uniqint pairmap uniq any);
 use Ref::Util 0.100 qw(is_ref is_plain_hashref);
@@ -205,6 +205,7 @@ sub evaluate_json_string ($self, $json_data, $schema, $config_override = {}) {
       exception => 1,
       errors => [
         JSON::Schema::Modern::Error->new(
+          depth => 0,
           keyword => undef,
           instance_location => '',
           keyword_location => '',
@@ -302,6 +303,7 @@ sub evaluate ($self, $data, $schema_reference, $config_override = {}) {
     schema_path => '',                  # the rest of the path, since the start of evaluation, or last $id or $ref
     effective_base_uri => $effective_base_uri, # resolve locations against this for errors and annotations
     errors => [],
+    depth => 0,
   };
 
   exists $config_override->{$_} and die $_.' not supported as a config override'
@@ -336,7 +338,6 @@ sub evaluate ($self, $data, $schema_reference, $config_override = {}) {
 
     $state = +{
       %$state,
-      depth => 0,
       initial_schema_uri => $schema_info->{canonical_uri}, # the canonical URI as of the start of evaluation, or last $id or $ref
       document => $schema_info->{document},   # the ::Document object containing this schema
       document_path => $schema_info->{document_path}, # the path within the document of this schema, as of the start of evaluation, or last $id or $ref
@@ -403,13 +404,19 @@ sub validate_schema ($self, $schema, $config_override = {}) {
   return $self->evaluate($schema, $metaschema_uri, $config_override);
 }
 
-sub get ($self, $uri) {
-  my $schema_info = $self->_fetch_from_uri($uri);
+sub get ($self, $uri_reference) {
+  my $schema_info = $self->_fetch_from_uri($uri_reference);
   return if not $schema_info;
   my $subschema = is_ref($schema_info->{schema}) ? dclone($schema_info->{schema}) : $schema_info->{schema};
   return wantarray ? ($subschema, $schema_info->{canonical_uri}) : $subschema;
 }
 
+sub get_document ($self, $uri_reference) {
+  my $schema_info = $self->_fetch_from_uri($uri_reference);
+  return if not $schema_info;
+  return $schema_info->{document};
+}
+
 # defined lower down:
 # sub add_vocabulary { ... }
 # sub add_encoding { ... }
@@ -789,6 +796,7 @@ sub _get_metaschema_info ($self, $metaschema_uri, $for_canonical_uri) {
         # absolute location is undef iff the location = '/$schema'
         my $absolute_location = $e->absolute_keyword_location // $for_canonical_uri;
         JSON::Schema::Modern::Error->new(
+          depth => $e->depth,
           keyword => $e->keyword eq '$schema' ? '' : $e->keyword,
           instance_location => $e->instance_location,
           keyword_location => ($for_canonical_uri->fragment//'').($e->keyword_location =~ s{^/\$schema\b}{}r),
@@ -954,7 +962,7 @@ sub _get_or_load_resource ($self, $uri) {
   return;
 };
 
-# returns information necessary to use a schema found at a particular URI:
+# returns information necessary to use a schema found at a particular URI or uri-reference:
 # - a schema (which may not be at a document root)
 # - the canonical uri for that schema,
 # - the JSON::Schema::Modern::Document object that holds that schema
@@ -963,12 +971,12 @@ sub _get_or_load_resource ($self, $uri) {
 # - the vocabularies to use when considering schema keywords
 # - the config overrides to set when considering schema keywords
 # creates a Document and adds it to the resource index, if not already present.
-sub _fetch_from_uri ($self, $uri) {
-  $uri = Mojo::URL->new($uri) if not is_ref($uri);
-  my $fragment = $uri->fragment;
+sub _fetch_from_uri ($self, $uri_reference) {
+  $uri_reference = Mojo::URL->new($uri_reference) if not is_ref($uri_reference);
+  my $fragment = $uri_reference->fragment;
 
   if (not length($fragment) or $fragment =~ m{^/}) {
-    my $base = $uri->clone->fragment(undef);
+    my $base = $uri_reference->clone->fragment(undef);
     if (my $resource = $self->_get_or_load_resource($base)) {
       my $subschema = $resource->{document}->get(my $document_path = $resource->{path}.($fragment//''));
       return if not defined $subschema;
@@ -993,7 +1001,7 @@ sub _fetch_from_uri ($self, $uri) {
     }
   }
   else {  # we are following a URI with a plain-name fragment
-    if (my $resource = $self->_get_resource($uri)) {
+    if (my $resource = $self->_get_resource($uri_reference)) {
       my $subschema = $resource->{document}->get($resource->{path});
       return if not defined $subschema;
       return {
@@ -1007,12 +1015,14 @@ sub _fetch_from_uri ($self, $uri) {
   }
 }
 
+use constant _JSON_BACKEND => Mojo::JSON::JSON_XS ? 'Cpanel::JSON::XS' : 'JSON::PP';
+
 # used for internal encoding as well (when caching serialized schemas)
 has _json_decoder => (
   is => 'ro',
   isa => HasMethods[qw(encode decode)],
   lazy => 1,
-  default => sub { JSON::MaybeXS->new(allow_nonref => 1, canonical => 1, utf8 => 1, allow_bignum => 1, convert_blessed => 1) },
+  default => sub { _JSON_BACKEND->new->allow_nonref(1)->canonical(1)->utf8(1)->allow_bignum(1)->convert_blessed(1) },
 );
 
 # since media types are case-insensitive, all type names must be casefolded on insertion.
@@ -1025,7 +1035,7 @@ has _media_type => (
     my $_json_media_type = sub ($content_ref) {
       # utf-8 decoding is always done, as per the JSON spec.
       # other charsets are not supported: see RFC8259 §11
-      \ JSON::MaybeXS->new(allow_nonref => 1, utf8 => 1)->decode($content_ref->$*);
+      \ _JSON_BACKEND->new->allow_nonref(1)->utf8(1)->decode($content_ref->$*);
     };
     +{
       (map +($_ => $_json_media_type),
@@ -1036,7 +1046,7 @@ has _media_type => (
         \ Mojo::Parameters->new->charset('UTF-8')->parse($content_ref->$*)->to_hash;
       },
       'application/x-ndjson' => sub ($content_ref) {
-        my $decoder = JSON::MaybeXS->new(allow_nonref => 1, utf8 => 1);
+        my $decoder = _JSON_BACKEND->new->allow_nonref(1)->utf8(1);
         my $line = 0; # line numbers start at 1
         \[ map {
             do {
@@ -1122,7 +1132,7 @@ JSON::Schema::Modern - Validate data against a schema
 
 =head1 VERSION
 
-version 0.577
+version 0.578
 
 =head1 SYNOPSIS
 
@@ -1646,9 +1656,16 @@ You can use it thusly:
   my $schema = $js->get($uri);
   my ($schema, $canonical_uri) = $js->get($uri);
 
-Fetches the Perl data structure representing the JSON Schema at the indicated URI. When called in
-list context, the canonical URI of that location is also returned, as a L<Mojo::URL>. Returns
-C<undef> if the schema with that URI has not been loaded (or cached).
+Fetches the Perl data structure representing the JSON Schema at the indicated identifier (uri or
+uri-reference). When called in list context, the canonical URI of that location is also returned, as
+a L<Mojo::URL>. Returns C<undef> if the schema with that URI has not been loaded (or cached).
+
+=head2 get_document
+
+  my $document = $js->get_document($uri_reference);
+
+Fetches the L<JSON::Schema::Modern::Document> object that contains the provided identifier (uri or
+uri-reference). C<undef> if the schema with that URI has not been loaded (or cached).
 
 =head1 LIMITATIONS
 
@@ -1661,7 +1678,7 @@ string is used in an arithmetic operation), additional flags can be added onto t
 it to resemble the other type. This should not be an issue if data validation is occurring
 immediately after decoding a JSON payload, or if the JSON string itself is passed to this module.
 If you are still having difficulties, make sure you are using Perl's fastest and most trusted and
-reliable JSON decoder, L<Cpanel::JSON::XS> (or its proxy, useful for fatpacking, L<JSON::MaybeXS>).
+reliable JSON decoder, L<Cpanel::JSON::XS>.
 Other JSON decoders are known to produce data with incorrect data types.
 
 For more information, see L<Cpanel::JSON::XS/MAPPING>.
