@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Error;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains a single error from a JSON Schema evaluation
 
-our $VERSION = '0.577';
+our $VERSION = '0.578';
 
 use 5.020;
 use Moo;
@@ -19,7 +19,7 @@ use Safe::Isa;
 use JSON::PP ();
 use MooX::TypeTiny;
 use Types::Standard qw(Str Bool Undef InstanceOf Enum Tuple);
-use Types::Common::Numeric 'PositiveInt';
+use Types::Common::Numeric qw(PositiveInt PositiveOrZeroInt);
 use namespace::clean;
 
 use overload
@@ -64,6 +64,12 @@ has recommended_response => (
   isa => Tuple[PositiveInt, Str],
 );
 
+has depth => (
+  is => 'ro',
+  isa => PositiveOrZeroInt,
+  required => 1,
+);
+
 sub TO_JSON ($self) {
   return +{
     # note that locations are JSON pointers, not uri fragments!
@@ -82,7 +88,12 @@ sub stringify ($self) {
 }
 
 sub dump ($self) {
-  my $encoder = JSON::MaybeXS->new(utf8 => 0, convert_blessed => 1, canonical => 1, indent => 1, space_after => 1);
+  my $encoder = JSON::Schema::Modern::_JSON_BACKEND()->new
+    ->utf8(0)
+    ->convert_blessed(1)
+    ->canonical(1)
+    ->indent(1)
+    ->space_after(1);
   $encoder->indent_length(2) if $encoder->can('indent_length');
   $encoder->encode($self);
 }
@@ -95,7 +106,7 @@ __END__
 
 =encoding UTF-8
 
-=for stopwords schema fragmentless
+=for stopwords schema fragmentless subschemas
 
 =head1 NAME
 
@@ -103,7 +114,7 @@ JSON::Schema::Modern::Error - Contains a single error from a JSON Schema evaluat
 
 =head1 VERSION
 
-version 0.577
+version 0.578
 
 =head1 SYNOPSIS
 
@@ -163,6 +174,11 @@ string to use for this error (if validating an HTTP request). This could exist f
 failed authentication check in OpenAPI validation, in which case it would contain
 C<[ 401, 'Unauthorized' ]>.
 
+=head2 depth
+
+An integer which indicates how many subschemas deep this error was generated from. Can be used to
+construct a tree-like structure of errors.
+
 =head1 METHODS
 
 =for Pod::Coverage stringify mode
