@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Result;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains the result of a JSON Schema evaluation
 
-our $VERSION = '0.577';
+our $VERSION = '0.578';
 
 use 5.020;
 use Moo;
@@ -47,7 +47,7 @@ has exception => (
   is => 'ro',
   isa => Bool,
   lazy => 1,
-  default => sub { any { $_->exception } $_[0]->errors },
+  default => sub ($self) { any { $_->exception or $_->error =~ /^EXCEPTION: / } $self->errors },
 );
 
 has $_.'s' => (
@@ -81,9 +81,7 @@ has recommended_response => (
       return $pe if $pe;
     }
 
-    return [ 500, 'Internal Server Error' ]
-      if any { $_->exception or $_->error =~ /^EXCEPTION: / } $self->errors;
-
+    return [ 500, 'Internal Server Error' ] if $self->exception;
     return [ 400, ($self->errors)[0]->stringify ];
   },
 );
@@ -210,7 +208,12 @@ sub TO_JSON ($self) {
 }
 
 sub dump ($self) {
-  my $encoder = JSON::MaybeXS->new(utf8 => 0, convert_blessed => 1, canonical => 1, indent => 1, space_after => 1);
+  my $encoder = JSON::Schema::Modern::_JSON_BACKEND()->new
+    ->utf8(0)
+    ->convert_blessed(1)
+    ->canonical(1)
+    ->indent(1)
+    ->space_after(1);
   $encoder->indent_length(2) if $encoder->can('indent_length');
   $encoder->encode($self);
 }
@@ -239,7 +242,7 @@ JSON::Schema::Modern::Result - Contains the result of a JSON Schema evaluation
 
 =head1 VERSION
 
-version 0.577
+version 0.578
 
 =head1 SYNOPSIS
 
