@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Utilities;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Internal utilities for JSON::Schema::Modern
 
-our $VERSION = '0.577';
+our $VERSION = '0.578';
 
 use 5.020;
 use strictures 2;
@@ -16,7 +16,6 @@ no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
 use B;
 use Carp 'croak';
-use JSON::MaybeXS 1.004004 'is_bool';
 use Ref::Util 0.100 qw(is_ref is_plain_arrayref is_plain_hashref);
 use Scalar::Util qw(blessed looks_like_number);
 use Storable 'dclone';
@@ -111,6 +110,14 @@ sub get_type ($value) {
   return 'ambiguous type';
 }
 
+# lifted from JSON::MaybeXS
+sub is_bool ($value) {
+  Scalar::Util::blessed($value)
+    and ($value->isa('JSON::PP::Boolean')
+      or $value->isa('Cpanel::JSON::XS::Boolean')
+      or $value->isa('JSON::XS::Boolean'));
+}
+
 # compares two arbitrary data payloads for equality, as per
 # https://json-schema.org/draft/2020-12/json-schema-core.html#rfc.section.4.2.2
 # if provided with a state hashref with a 'path' key, any differences are recorded within
@@ -215,6 +222,7 @@ sub canonical_uri ($state, @extra_path) {
 # - errors
 # - exception (set by abort())
 # - recommended_response
+# - depth
 sub E ($state, $error_string, @args) {
   croak 'E called in void context' if not defined wantarray;
 
@@ -229,6 +237,7 @@ sub E ($state, $error_string, @args) {
     or ($uri->fragment // '') eq $keyword_location and $uri->clone->fragment(undef) eq '';
 
   push $state->{errors}->@*, JSON::Schema::Modern::Error->new(
+    depth => $state->{depth},
     keyword => $state->{keyword},
     instance_location => $state->{data_path},
     keyword_location => $keyword_location,
@@ -253,6 +262,7 @@ sub E ($state, $error_string, @args) {
 # - collect_annotations
 # - spec_version
 # - _unknown
+# - depth
 sub A ($state, $annotation) {
   return 1 if not $state->{collect_annotations} or $state->{spec_version} eq 'draft7';
 
@@ -266,6 +276,7 @@ sub A ($state, $annotation) {
     .jsonp($state->{schema_path}, $state->{keyword}, delete $state->{_schema_path_suffix});
 
   push $state->{annotations}->@*, {
+    depth => $state->{depth},
     keyword => $state->{keyword},
     instance_location => $state->{data_path},
     keyword_location => $keyword_location,
@@ -372,7 +383,7 @@ JSON::Schema::Modern::Utilities - Internal utilities for JSON::Schema::Modern
 
 =head1 VERSION
 
-version 0.577
+version 0.578
 
 =head1 SYNOPSIS
 
@@ -382,7 +393,7 @@ version 0.577
 
 This class contains internal utilities to be used by L<JSON::Schema::Modern>.
 
-=for Pod::Coverage is_type get_type is_equal is_elements_unique jsonp unjsonp local_annotations
+=for Pod::Coverage is_type get_type is_bool is_equal is_elements_unique jsonp unjsonp local_annotations
 canonical_uri E A abort assert_keyword_exists assert_keyword_type assert_pattern assert_uri_reference assert_uri
 annotate_self sprintf_num
 
